GET STARTED
=
1. Download and unzip the latest version of MapWall
2. Install a web server and expose your mapwall directory (if you don't have one yet)
    ```
    npm install --global http-server
    cd mapwall-master && http-server
    /* go to http://localhost:8080/tpl_1_map_minimal.html from your browser */
    ```
3. Optional but strongly recommended for newbies: start coding your application from one of our templates
    - **tpl_1_map_minimal.html** minimal single map application, good for starting from scratch
    - **tpl_2_maps_minimal.html** like the previous one but with a double map layout
    - **tpl_3_maps_fullopt.html** 3 maps layout with almost all the MapWall features deeply explained in comments, intended to be used as an how-to sample
    - **tpl_4_maps_resize.html** 4 maps layout with resizable viewports
5. Edit the configuration json file defined in the data-config attribute of mapwall.js's inclusion script tag
6. Place your procedures for getting GIS data after the mapwallReady event has been dispatched
    ```
    document.addEventListener('mapwallReady', async (event) => {
      //your code
    });
    ```
7. Add your GIS data to the map using the addLayer method from the MapWall API
    ```    
    await mapwall.getMap("myMapId").addLayer(myLayerObject, "myLayerName")
    ```
8. Code your application logic and Mapwall's callbacks (such as fillWatcher) for placing contents into watcher or panels


JSON Configuration file
=
The configuration file is specified in the `data-config` attribute of the mapwall.js inclusion script tag. Here a snippet:
```
{
  {
  "geo": {
    "mastermap": "all",
    "interaction": {
      "event": {
        "mode":"multi"
      }
      "sharedView": {
        "startLinked":true,
        "center":{
          "lat":40.086493, 
          "lon":9.089684
        },
        "zoom":8,
        "maxZoom":18,
        "minZoom":7
      }
    },
    "maps": []  
  }
  "links": [
      {"label": "CRS4", "url": "http://www.crs4.it", "classIcon":"fa fa-link"}
  ]    
}
```

`geo` Contains the view settings for the gis client. Mapwall can be configured with a single screen-wide map or with multiple maps, each of them defined in the *maps* array.

`geo/interaction/sharedView` The sharedView properties such as *center* (coordinates in EPSG:4326) will be used to build the default View for the map, if it is missing in the maps array. The *startLinked* attribute is used in multi-map aplications: using startLinked == true all the map will move together.

`geo/interaction/event` In multimap applications if *mode* is set to "multi", mapwall forwards the click event to each map at the same coordinate, showing the relative popup or executing the defined callback in that point.
 
`geo/maps` An array containing the map view definitions, such as attributions, base layer type and source ecc. The ID for the map is mandatory en it is essential for addressing the map in your code.
```
{
  "id":"mw_map_1",
  "view": {
    "center":{
      "lat":39.168,
      "lon":9.03
    }
  },
  "attribution": "map data: © <a href='https://openstreetmap.org/copyright'>OpenStreetMap</a> contributors",
  "basemap": {
    "src": "https://c.tile.opentopomap.org/{z}/{x}/{y}.png",
    "type": "XYZ",
    "className":"css-class-name"
  }
}
```

`geo/maps//id` A unique id for the map, if you define a custom layout in your html, make sure that the map container (div) has this id as element id

`geo/maps//basemap` Optional field, if not present mapwall shows the standard OSM basemap by default. It is useful to define a custom background tiled map in place of the default OSM one. It supports WMS, OSM and XYZ openlayers source types for _Tile_ layers. For the OSM and XYZ the _src_ must be an xyz URL template: *http://osm.tileserver.org/hot/{z}/{x}/{y}.png*. For the WMS type you can specify all the protocol parameters in the _src_ URL. Use _attribution_ to define the attributions string for the map, if not provided the OSM standard one will be used for the OSM baselayer.

`links` array of url shown by clicking on the control menu *links* button, if enabled (see buttonBar below):
```
"links": [
  {"label": "CRS4", "url": "http://www.crs4.it", "classIcon":"icon icon-class"}
]
```

`buttonBar` The application button bar definition (round buttons below). **draggable** allows mouse dragging of the button bar over the map. **moveOnDoubleClick** enables the instant move on the double-clicked position. For each element in the **items** section, only the unique ID of the button, the icon class and the callback are mandatory. **iconClass** is a CSS icon from your favorite (and previously included in your page) css iconset such as fontawesome. Set **visible** to _false_ to hide the corresponding button. **callback** defines the callback to execute on click. **active** field enable the active background when the button is clicked. If _active_ is true or if you call _toggleButtonActive()_ the icon will change to **activeIconClass**, if set. If **lockable** is true the button will be disabled (enabled) when calling the _toggleLockableButtons()_ Mapwall API (note: at startup all buttons are enabled by default), very useful in editing or state changing scenarios. **lockIfActive** false for a _lockable_ button prevent the button to be locked by _toggleLockableButtons_ when it's in active state. Use **classlist** to customize the button style through your CSS classes (space-separated list). If **label** is set, a label appears on top of the button when the mouse pointer is over it.
To create new custom buttons just add records to the **items** array. You can either add a new button or customize the default ones. The buttonBar configuration of Mapwall's template apps provides common use functionalities already defined within the library.
```
"buttonBar": {
  "orientation":"horizontal",
  "align":"center",
  "draggable":true,
  "moveOnDoubleClick":false,
  "items": [
    {
      "id": "homeLink", 
      "visible":true, 
      "active":false, 
      "lockable": false,
      "lockIfActive": false,
      "callback":"mapwall.goHome", 
      "iconClass": "fa fa-home", 
      "activeIconClass": "fa fa-close",
      "classlist": "myBtnClass1 myBtnClass2",
      "label": "Button label"
    },
    ...
  ]
}
```

`panels` an array of objects where you can define the rendering properties of *fixed overlay panels*
shown over the map. You can use them to show application-wide data, not related to a specific feature of the map.

```
"panels": [
  {
    "baseId":"topboxes",
    "callback":"topboxesHandler",
    "resizable":false,
    "spacing":30,
    "classes":"class1 class2",
    "instances":4,
    "size": {
      "width":200, 
      "height":100
    },
    "customSize": {
        "1": {
            "width":200,
            "height":300
        }
    },
    "position": {
      "vOffset": 0,
      "hAlign": "left",
      "vAlign": "top",
      "orientation":"horizontal"
    }
  }
]
```
`panels/callback` executed one time for each panel at startup **before the mapwallReady event**! 
 It takes one parameter representing the panel element ID, which is formed by the panel group _baseId_, plus _ (underscore)
 and a progressive number counting from the **halign** (in h orientation) side you defined 

`panels/spacing` the margin between panels in the panel group

`panels/classes` CSS classes to be applied to each panel

`panels/instances` the number of panels in this group

`panels/customSize` if you need a different size for any panel in the panel group, use its position-id as key in this object (see the snippet above)

`panels/position/vOffset`is the a margin from the top of the page in pixel

`panels/position/halign` _right_ means panels on the right of the page **and** rightToLeft reading, so panel 1 is the rightmost one

`panels/position/valign` _top_ means panels on top of the page **and** top-down reading, so panel 1 is the topmost one

`panels/position/orientation` _horizontal_ means that the panel group is disposed horizontally, the other option is _vertical_

`slidingpanels` like *panels* this is an optional array of objects where you can define the rendering properties of *fixed overlay panels* that slide outside/inside the screen
```
"slidingPanels": [
  {
    "id":"settingsBox",
    "callback":"settingsHandler",
    "classes":"",
    "iconCss": "fa fa-sliders",
    "hidden":true,
    "mouseover":true,
    "size": {
      "width":350, 
      "height":500
    },
    "position": {
      "top": 30,
      "align": "right"
    }
  }
]
```

`popupCallback` *DEPRECATED* The **name** (string) of the **default** callback that will be called on each popup of your 
application if you don't define any other in the _addLayer_ method (see API below). Use this function 
to enable custom actions on your popups and fill their dynamic content. 
It takes two arguments:
- the popup id (matches the related feature id)
- the popup DOM element


Helpers
=
**Always set a unique id for your features**, it is used as identifier on each callback. If no id is provided Mapwall will set a random Id for you. 
The ID must be unique for the whole application, no other features, even in different layers or maps should have the same id.

### MapwallWMSTileLayer
Mapwall class for easily declaring a WMS tiled layer. The constructor requires the WMS source URL and an optional string containing the style name for the map. The second parameter can be also set as an object containing the **ol.layer.Tile** [constructor options](https://openlayers.org/en/latest/apidoc/module-ol_layer_Tile-TileLayer.html), in this case you can define the map style by adding a _style:"styleName"_ property to the object.

`MapwallWMSTileLayer.getLayer()` Returns the Openlayers **ol.layer.Tile** layer object created by the helper

```
/* Example */
let layer = new MapwallWMSTileLayer(URL)
await mapwall.getMap().addLayer(layer, "myLayer")
```

### MapwallWFSGeoJsonLayer
Mapwall class for easily declaring a WFS layer exposed in GeoJSON format. The constructor requires the WFS source URL and an optional Openlayers style object or function for the features. The second parameter can be also set as an object containing the **ol.layer.Vector** [constructor options](https://openlayers.org/en/latest/apidoc/module-ol_layer_Vector-VectorLayer.html). To be sure that your features have a unique ID, even between multiple WFS layers you can define the **_idPrefix_** option in the second parameter; if defined, the WFS _ogc_fid_ default ID will be prefixed with this string for each feature. WFS parameters can be defined both in the url or within the _params_ optional argument. If you choose the latter you must add a _wmsParams_ object containing the WFS parameters as an object.

`MapwallWFSGeoJsonLayer.getLayer()` Returns the Openlayers **ol.layer.Vector** layer object created by the helper

```
/* Example */
let layer = new MapwallWFSGeoJsonLayer(URL)
await mapwall.getMap().addLayer(layer, "myLayer")
```

### MapwallVectorLayer
Mapwall class for easily declaring a _ol.layer.Vector_ layer. The constructor takes an array of _ol.Feature_ or _MapwallFeatureWKT_ and an optional Openlayers style object or function for the features. The second parameter can be also set as an object containing the **ol.layer.Vector** [constructor options](https://openlayers.org/en/latest/apidoc/module-ol_layer_Vector-VectorLayer.html).

`MapwallVectorLayer.getLayer()` Returns the Openlayers **ol.layer.Vector** layer object created by the helper

```
/* Example */
let feature = new MapwallFeatureWKT("POINT(12.4923 41.8902)")
let layer = new MapwallVectorLayer([feature])
await mapwall.getMap().addLayer(layer, "myLayer")
```

### MapwallFeatureWKT
Mapwall class for simplifying an _ol.Feature_ declaration when its geometry is in WKT format. The constructor takes a WKT geometry an two optional parameters for SRS transformation: the source SRS and destination SRS. **Keep in mind** that transformations between SRSs different from 4326 and 3857 need the [proj4js library](https://github.com/proj4js/proj4js), see [the Openlayers FAQ](https://openlayers.org/en/latest/doc/faq.html#how-do-i-change-the-projection-of-my-map-).

`MapwallFeatureWKT.setId(ID)` set the object's ol.Feature id

`MapwallFeatureWKT.getId()` return the id of the object's ol.Feature (if set)

`MapwallFeatureWKT.set(name, value)` set an attribute in the object's ol.feature

`MapwallFeatureWKT.get(name)` get an object's ol.Feature attribute

```
/* Example */
let feature = new MapwallFeatureWKT("POINT(12.4923 41.8902)", "EPSG:4326", "EPSG:3857")
```

API
=
Mapwall APIs refer to the UI interaction. 
MWMap APIs are for interacting with the map.

Most of the methods below and their correct usage are described in depth in the comments of each html template file. 
`[]` means *optional*. **(v)** methods are for Vector layers only.

### Mapwall API

`getMap(map_id)` return the Mapwall _MWMap_ object corresponding to the ID passed as parameter, in single map applications map_id is not required

`getOpenWatchers([callback])` - return the list of DOM elements representing the visible feature watchers

`getUpdatableWatchers([callback])` - return the list of DOM elements representing the visible and updatable feature watchers (those without _.mw_disableupdate_ class)

`activateWarningBox(message, [confirm, callback])` - Shows a warning box on top of the page. If the confirm parameter is set to true two buttons are shown: YES and NO. By clicking on YES mapwall executes the callback defined in the third parameter; NO only closes the warning box.

`deactivateWarningBox()` - closes an open Warning Box

`toggleSharedView` Enable/disable sharedview in a multimap application, based on the interaction/sharedView global property.

`toggleLastSharedView` Enable/disable sharedview in a multimap application, using the last moved map.

`getMapList()` - In multimap applications returns an array containing map ids

`closeWatchers([watcherId])` - closes the watcher with the specified id, or all the open watchers

`closeMapWatchers(mapId)` - closes all the open watchers of the specified map

`toggleAppMenu()` - shows the application menu over the _buttonBar_

`toggleLockableButtons([lock])` - Lock/unlock _buttonBar_ buttons marked as _lockable_ in configuration. It adds a _data-mw-locked_ attribute to the button DOM element. The optional boolean parameter _lock_ forces to lock or unlock the button regardless of the _data-mw-locked_ value.

`toggleButtonActive(buttonId, [active])`- toggles active/disabled the button _buttonId_ in che buttonBar. If the optional parameter active is _true/false_ the button is forced to active/disabled.

`toggleToolBarMoving()` - Disable/enable toolbar movement over the screen on double click. This only works if the _moveOnDoubleClick_ in the configuration file is set to true. It can sometimes be useful to place the call in a timeout callback, especially when used within other events.

`freezeMapClick(freeze)` - suspend/re-enable Mapwall's click event listeners. Useful in OL interaction states or in case of events overwriting. If _freeze_ is true Mapwall listeners are blocked.

`spinnerOn()` - lock the screen by showing a rotating spinner, useful for waiting for long tasks

`spinnerOff()` - unlock the screen 


### MWMap API

**VERY IMPORTANT: if you set your feature IDs explicitly, they must be unique in the whole application, even between different layers or maps.**

`addLayer(layer, name, [pars], [callback]) **ASYNC**` This is the core function of Mapwall. It adds an Openlayers layer object (Vector or Tile) to the map. _layer_ can be both an ol.layer (Tile or Vector) object or a Mapwall helper. **IMPORTANT NOTE:** Mapwall helpers may need asynchronous initialization, so remember to call addLayer under await or sync your code through the optional _callback_ (see note below) when using them. _pars_ is an optional object with these fields: 
```
{
  callback:'functionName'|function,
  watcher:"HTML string"|DOM Element,
  zindex:N,
  filter:'functionName'|function,
  placemarker:true|false
}
``` 
The following parameters are optional:

- **watcher** DOM Element or an HTML string where you define a custom popup that is shown on click over a map feature or coordinate. In runtime mapwall adds the attributes listed below to the root element of the popup, which is then passed to the *fillWatcher* method (see below):
  - **data-feature-type**: the name of the layer to which the feature belongs
  - **data-feature-id**: the clicked feature ID
  - **data-map-id**: the id of the map
  - **data-click-coordinate**: the coordinate (lon/lat) of the point of the map where the user did click
- **filter** Function to be executed just before the popup is shown. It takes the map click event with the addiction of a *mapwall_data* property, containing the feature_id and the layer name. It is primarly intended to perform "before" operations needed for restricting the watcher opening, especially on Tile layers, where you don't have "built-in" feature identifiers. The *filter* callback must return a boolean or a string: if _false_ the watcher will not open, if string the feature (and watcher) ID will be set to that string; be careful with Vector layers because returning a string means that the watcher will be "unlinked" from the clicked feature, this behaviour is allowed mainly to link one watcher to a cluster of features. **The *filter* function is called under *await***, therefore you can use promises in your implementation.
- **zindex** Position of the layer in the OpenLayer's map stack.
- **callback** Custom callback name or object reference to be executed on click over a layer's feature or point. __If both *callback* and *watcher* are set then *watcher* is shown but filled using *callback* in place of the default *fillWatcher*__ (replaces deprecated popupCallback config property). If watcher is not defined, this function is executed in place of the default watcher visualization routine (no automatic watcher is shown). It takes 3 arguments: *feature ID*, *event*, *featureType*; If the listening layer is a Tile layer **and watcher is not set**, the *feature ID* is a coordinate object {lat:YY.YYY, lon:XX,XXX}. 
- **placemarker** if true, mapwall shows a place marker where the user clicks on a *Tile* layer (**only when a watcher is shown**)

**ADDLAYER CALLBACK NOTE: pay attention to the difference between the _layer_ callback and the _addLayer_ callback:**
```
mapwall.getMap().addLayer(
  layer, 
  "layername", 
  { 
    callback: () => {
      /**
       * layer callback executed on click on a feature or point
       */
    }
  }, 
  (layer) => {
    /**
     * addLayer callback, executed when addLayer completes operations. 
     * Convenient to keep async the init task for heavy layers (use in place of await)
     */
  }
})
```

`removeLayer(layerName)` - delete a layer from the map

`setListeningLayers(layerNames)` - set the listening layers, it takes a list (Array) of names or *undefined* to reset (see paragraph about listening layers below)

`getListeningLayers()` - return the name list of the listening layers set with setListeningLayers

`enableListeningLayers()`

`disableListeningLayers()`

`getLayersNames([callback])` - return the list of the layer names you previously set with addLayer

`getLayerByName(name, [callback])` - return the OL *layer* object identified through the name you gave him in addLayer

`getSourceByLayerName(name, [callback])` - return the OpenLayers *Source* of the layer *layerName*

`toggleLayer(layerName, [show])` - Show a layer if hidden or hide it if visible. Return true if the layer is set to visible, false otherwise. If the optional parameter is _true/false_ the layer is forced to visible/invisible

`getMap()` - return the OpenLayers _ol.Map_ object to gain access to all the OL functionalities

`moveTo(coords, [optargs])` - Moves the map view to the specified coordinates and zoom on. `coords` must be an array containing *[lon, lat]* in this order. If their projection system is different from 3857, you must specify it in the optargs object ({epsg:4326}... see https://openlayers.org/en/latest/doc/faq.html#how-do-i-change-the-projection-of-my-map-) where you can also define a zoom level ({zoom:12})

`setView(view)` - set a new _ol.View_ for the map

`getView()` - returns the _ol.View_ object of the Openlayers map

`getFeatureById(layerName, featureId, [callback])` - get the OL feature object identified by *featureId* and belonging to the *layerName* layer, or *null*

`addFeatureToLayer(layerName, feature, [callback])` - add a new feature to the *layerName* layer; for vector layers if no ID is set for the new feature, a timestemp is set as default ID. For Tile layers the feature id is mandatory and one of *id* or *mwid* property must be present in the *feature* object. 
***For Tile layers*** *feature* can be an arbitrary object, not an ol.Feature instance. Due to the nature of Tile layers, the definition of a feature and its behaviour is up to you and should be handled using the *filter* method passed to *addLayer*. 
The basic idea for using this method with tile layers is to emulate the behaviour of a Vector layer; the typical steps should be:
- get the WMS or pixel informations in the filter method
- create the "feature" object and add to the layer with addFeatureToLayer
- get the information with layer.getFeatureById() in the watcher callback (fillWatcher)

`removeFeatureFromLayer(layerName, featureID, [callback])` - remove a feature from the *layerName* layer.

`addFeaturesToLayer(layerName, featuresArray, [callback])` - add a batch of feature to the *layerName* layer

`changeBaseLayer(id)`- Switches beetween the map baselayers declared in the configuration json. Passing an id not specified in the basemap array of the configuration json or an array index out of range results in a blank baselayer.

`toggleBaseLayer()`- Shows or hides the map base layer (typically OSM)

(v) `openFeatureWatcher(feature, [move])` - Opens a vector-feature's watcher manually, without phisically click on it. Feature is a _ol.Feature_ object, the optional boolean _move_ parameter makes the map zoom on _feature_. 

(v) `blinkFeature(featureId, layerName)` - Makes a feature blinking on the map. Works with Vector layers only

(v) `toggleFeatures(layerName, [propertyName, propertyValue])` - Toggle features with *propertyName == propertyValue* and belonging to *layerName*. 
If propertyName and propertyValue are not set the action applies to every layer feature. Keep in mind that toggle means hidden become visible and visible hidden.

(v) `toggleFeaturesEval(layerName, evaluate)`- Same as _toggleFeatures_ but with condition expressed through your evaluate function which takes a feature as argument
```
/**
 * Toggles all the features whose id doesn't match "myId"
 * Example valid for hideFeaturesEval and showFeaturesEval too
 */
mapwall.getMap().toggleFeaturesEval("myLayer", (feature) => {return feature.getId != "myId"});

```

(v) `hideFeatures(layerName, [propertyName, propertyValue])` - Hide features with *propertyName == propertyValue* and belonging to *layerName*. 
If propertyName and propertyValue are not set the action applies to every layer feature

(v) `hideFeaturesEval(layerName, evaluate)`- Same as _hideFeatures_ but with condition expressed through your evaluate function which takes a feature as argument (see toggleFeaturesEval example above)

(v) `showFeatures(layerName, [propertyName, propertyValue])` - Show features with *propertyName == propertyValue* and belonging to *layerName*. 
If propertyName and propertyValue are not set the action applies to every layer feature

(v) `showFeaturesEval(layerName, evaluate)`- Same as _showFeatures_ but with condition expressed through your evaluate function which takes a feature as argument (see toggleFeaturesEval example above)

Listening Layers
=
If your application's map uses more than one layer (tile or vector) you need to define how Mapwall must handle callbacks. Layers are disposed in a stack, by default Mapwall executes the callback associated to the topmost one with values in the clicked point, to impose a different behaviour you should use the *setListeningLayers* API through which you can: 
- define a different target layer: for example *setListeningLayers(["layer2"])* will execute the layer2 callback instead of the layer1 which is on top.
- define multiple callback execution: for example *setListeningLayers(["layer2", "layer3"])* will execute the layer2 and layer3 callback but not the layer1's one.

In the case of multiple layers in the setListeningLayers argument array, the execution order of the callbacks will be the same of the elements in that array.


Special CSS classes
=
You can control many mapwall's behaviours using special CSS classes:

`draggable` Add this class to the elements you want to be draggable by the user

`resizable` Add this class to the elements you want to be resizable by the user

`mw_donotrestore` Inhibits auto resize of panels on `restoreWindow()` 

`mw_disabledrag` Disable popup/panel drag gesture over the element 

`mw_disableupdate` Set on a popup element remove it from the `getUpdatableWatchers()` list


LICENSE AND ATTRIBUTION
=
MapWall is developed and mantained by the DTA group at CRS4 Research Center (www.crs4.it). It is released under MIT license.
We would really appreciate if you could show this small banner on your application (already present in template files)

`<div class="attrib"><img src="mapwall-logo.png"><span><a href="http://mapwall.crs4.it">MapWall</a> by CRS4</span></div>`


CREDITS
=
MapWall is built on top of these great Open projects:

- [OpenLayers](https://openlayers.org/): a powerful, fully featured webgis client
- [interactjs](https://interactjs.io/): drag and drop, resizing, and multi-touch gestures for modern browsers

